const std = @import("std");
const util = @import("util.zig");

const numbers = [_][]const u8{ "zero", "one", "two", "three", "four", "five", "six", "seven", "eight", "nine" };

pub fn main() !void {
    const input = @embedFile("data/day01.txt");
    std.debug.print("{d}\n", .{try solve(input, &[_][]u8{})});
    std.debug.print("{d}\n", .{try solve(input, &numbers)});
}

const zero = @as(u8, '0');

fn solve(input: []const u8, words: []const []const u8) !u32 {
    var sum: u32 = 0;

    var it = std.mem.splitAny(u8, input, "\n");
    while (it.next()) |line| {
        if (std.mem.eql(u8, "", line)) {
            continue;
        }

        var first: u8 = 0;
        var last: u8 = 0;

        for (0..line.len) |offset| {
            var n: u8 = 0; // ascii value
            if (line[offset] >= '0' and line[offset] <= '9') {
                n = line[offset];
            } else {
                for (words, 0..) |word, value| {
                    if (std.mem.startsWith(u8, line[offset..], word)) {
                        n = @as(u8, @intCast(value)) + zero;
                        break;
                    }
                }
            }
            if (n != 0) {
                if (first == 0) {
                    first = n;
                }
                last = n;
            }
        }

        const cal_value = (first - zero) * 10 + (last - zero);
        sum += cal_value;
    }

    return sum;
}

test "silver" {
    const input =
        \\1abc2
        \\pqr3stu8vwx
        \\a1b2c3d4e5f
        \\treb7uchet
    ;
    try std.testing.expectEqual(@as(u32, 142), try solve(input, &[_][]u8{}));
}

test "gold" {
    const input =
        \\two1nine
        \\eightwothree
        \\abcone2threexyz
        \\xtwone3four
        \\4nineeightseven2
        \\zoneight234
        \\7pqrstsixteen
    ;
    try std.testing.expectEqual(@as(u32, 281), try solve(input, &numbers));
}
